'****************************************************
'* PAINT3D.BAS Version 1.0 Date: 03/01/95           *
'* VB Tips & Tricks                                 *
'* 8430-D Summerdale Road San Diego CA 92126-5415   *
'* Compuserve: 74227,1557                           *
'* America On-Line: DPMCS                           *
'* InterNet: DPMCS@AOL.COM                          *
'*==================================================*
' The two most important function of this module are:
'
'       FormIn3D            Draws a form and its controls in 3D.
'                           Also initialize CTL3D.DLL.
'                           Call it from each forms Paint event.
'
'       Unregister_Ctl3D    Clean up call to CTL3D.DLL.
'                           Call it only when your application is closed.
'
' To draw Dialog in 3D, you got two methodes:
'
'       Dlg3DTimer          In each Dialog use a Timer control (Interval=1).
'                           Call this function from each timers Timer event.
'
'       Dlg3DBlaster        In each Dialog use a MsgBlaster control (Form
'                           messages to trap (eat): WM_NCPAINT, WM_NCACTIVATE).
'                           Call it from each MsgBlasters Message event.
'

Option Explicit


Declare Function GetActiveWindow Lib "User" () As Integer

Declare Function GetWindowDC Lib "User" (ByVal hWnd As Integer) As Integer
Declare Function ReleaseDC Lib "User" (ByVal hWnd As Integer, ByVal hDC As Integer) As Integer

Declare Function CreatePen Lib "GDI" (ByVal nPenStyle As Integer, ByVal nWidth As Integer, ByVal crColor As Long) As Integer
Declare Function SelectObject Lib "GDI" (ByVal hDC As Integer, ByVal hObject As Integer) As Integer
Declare Function DeleteObject Lib "GDI" (ByVal hObject As Integer) As Integer

Declare Function MoveTo Lib "GDI" (ByVal hDC As Integer, ByVal X As Integer, ByVal Y As Integer) As Long
Declare Function LineTo Lib "GDI" (ByVal hDC As Integer, ByVal X As Integer, ByVal Y As Integer) As Integer


'Pen Styles
Global Const PS_SOLID = 0
Global Const PS_DASH = 1        '  -------
Global Const PS_DOT = 2         '  .......
Global Const PS_DASHDOT = 3     '  _._._._
Global Const PS_DASHDOTDOT = 4  '  _.._.._
Global Const PS_NULL = 5
Global Const PS_INSIDEFRAME = 6

'Colors
Global Const COLOR_BLACK = &H0&
Global Const COLOR_LIGHT_GRAY = &HC0C0C0
Global Const COLOR_DARK_GRAY = &H808080
Global Const COLOR_WHITE = &HFFFFFF

'Misc Consts
Global Const dlgBorderWidth = 6
Global Const dlgCaptionHeight = 18
Global Const dlgSysMenuWidth = 18



' Types, Declares and Const needed for Dlg3DBlaster
Type typRect
    Left As Integer
    Top As Integer
    right As Integer
    bottom As Integer
End Type

Declare Function SetTextAlign Lib "GDI" (ByVal hDC As Integer, ByVal wFlags As Integer) As Integer
Declare Function SetTextColor Lib "GDI" (ByVal hDC As Integer, ByVal crColor As Long) As Long
Declare Function SetBkMode Lib "GDI" (ByVal hDC As Integer, ByVal nBkMode As Integer) As Integer
Declare Function GetStockObject Lib "GDI" (ByVal nIndex As Integer) As Integer
Declare Function TextOut Lib "GDI" (ByVal hDC As Integer, ByVal X As Integer, ByVal Y As Integer, ByVal lpString As String, ByVal nCount As Integer) As Integer

Declare Function CreateSolidBrush Lib "GDI" (ByVal crColor As Long) As Integer
Declare Function FillRect Lib "User" (ByVal hDC As Integer, lpRect As typRect, ByVal hBrush As Integer) As Integer
Declare Function Rectangle Lib "GDI" (ByVal hDC As Integer, ByVal X1 As Integer, ByVal Y1 As Integer, ByVal X2 As Integer, ByVal Y2 As Integer) As Integer

Declare Function GetSysColor Lib "User" (ByVal nIndex As Integer) As Long

Global Const WM_NCACTIVATE = &H86
Global Const WM_NCPAINT = &H85

Global Const TA_CENTER = 6
Global Const TA_TOP = 0

Global Const COLOR_ACTIVECAPTION = 2
Global Const COLOR_CAPTIONTEXT = 9
Global Const COLOR_INACTIVECAPTION = 3
Global Const COLOR_INACTIVECAPTIONTEXT = 19

Global Const TRANSPARENT = 1

Global Const SYSTEM_FONT = 13

'
' ComboBoxIn3D paints a 3D-border around the ComboBox given in ctrlCombo.
' frmTarget is the Form containing that ComboBox.
' If it is a Dropdown ComboBox, the ugly white space between the text field
' and the Dropdown-button is painted in lightgray.
'
'
Sub ComboBoxIn3D (ctrlCombo As Control, nBevel As Integer)
    Dim PixelX As Integer, PixelY As Integer
    Dim CTop As Integer, CRight As Integer, CBottom As Integer

    ' Just put "No 3D" in the Tag property and your ComboBox keeps 2D
    If InStr(UCase(ctrlCombo.Tag), "NO 3D") = 0 Then

	ControlIn3D ctrlCombo, nBevel, 0, True

	If ctrlCombo.Style = 0 Then             'Remove white space only
	    PixelX = Screen.TwipsPerPixelX      'if it is a Dropdown ComboBox
	    PixelY = Screen.TwipsPerPixelY
	    CTop = ctrlCombo.Top
	    CRight = ctrlCombo.Left + ctrlCombo.Width
	    CBottom = ctrlCombo.Top + ctrlCombo.Height
	    ctrlCombo.Parent.Line (CRight - PixelX * 24, CTop)-(CRight - PixelX * 18, CBottom - PixelY), COLOR_LIGHT_GRAY, BF
	End If
    End If

End Sub

'
' ControlIn3D paints a 3D-border around the control given in ctrlTarget.
' nBevel controls the the deepness, nSpace the distance between the control
' and the 3D-border and bInset sets the border to be drawn inset or outset.
'
' Parts of this code are taken from the VB Tips & Tricks help file.
' Original code written by Matej Nastran.
'
'
Sub ControlIn3D (ctrlTarget As Control, nBevel As Integer, nSpace As Integer, bInset As Integer)
    Dim CTop As Integer, CLeft As Integer, CRight As Integer, CBottom As Integer
    Dim PixelX As Integer, PixelY As Integer, AddX As Integer, AddY As Integer
    Dim i As Integer

    ' Just put "No 3D" in the Tag property and your control keeps 2D
    If InStr(UCase(ctrlTarget.Tag), "NO 3D") = 0 Then
	PixelX = Screen.TwipsPerPixelX
	PixelY = Screen.TwipsPerPixelY
	CTop = ctrlTarget.Top - PixelY
	CLeft = ctrlTarget.Left - PixelX
	CRight = ctrlTarget.Left + ctrlTarget.Width
	CBottom = ctrlTarget.Top + ctrlTarget.Height
	If bInset Then          ' Draw border inset
	    For i = nSpace To (nBevel + nSpace - 1)
		AddX = i * PixelX: AddY = i * PixelY
		ctrlTarget.Parent.Line (CLeft - AddX, CTop - AddY)-(CRight + AddX, CTop - AddY), COLOR_DARK_GRAY
		ctrlTarget.Parent.Line (CLeft - AddX, CTop - AddY)-(CLeft - AddX, CBottom + AddY), COLOR_DARK_GRAY
		ctrlTarget.Parent.Line (CLeft - AddX, CBottom + AddY)-(CRight + AddX + PixelX, CBottom + AddY), COLOR_WHITE
		ctrlTarget.Parent.Line (CRight + AddX, CTop - AddY)-(CRight + AddX, CBottom + AddY), COLOR_WHITE
	    Next i
	Else                    ' Draw border outset
	    For i = nSpace To (nBevel + nSpace - 1)
		AddX = i * PixelX: AddY = i * PixelY
		ctrlTarget.Parent.Line (CRight + AddX, CBottom + AddY)-(CRight + AddX, CTop - AddY), COLOR_DARK_GRAY
		ctrlTarget.Parent.Line (CRight + AddX, CBottom + AddY)-(CLeft - AddX, CBottom + AddY), COLOR_DARK_GRAY
		ctrlTarget.Parent.Line (CRight + AddX, CTop - AddY)-(CLeft - AddX - PixelX, CTop - AddY), COLOR_WHITE
		ctrlTarget.Parent.Line (CLeft - AddX, CBottom + AddY)-(CLeft - AddX, CTop - AddY), COLOR_WHITE
	    Next i
	End If
    End If

End Sub

'
' Dlg3DBlaster checks if Form is active and then draw the Non-Client area
' of the Form. Msg, wParam and lParam should be passed directly.
'
' Call this function in each forms MsgBlaster Message event.
'
'
Sub Dlg3DBlaster (frmTarget As Form, Msg As Integer, wParam As Integer, lParam As Long)
    Dim bActive As Integer

    'Check if Form is active window
    If Msg = WM_NCACTIVATE Then
	If wParam Then
	    bActive = True
	Else
	    bActive = False
	End If
    Else
	If frmTarget.hWnd = GetActiveWindow() Then
	    bActive = True
	Else
	    bActive = False
	End If
    End If

    DrawDlg3D frmTarget, True, bActive

End Sub

'
' Dlg3DFrameTimer checks if the 3D-border has to be repainted.
' bHasFocus must be a static variable in the Timer-event procedure.
'
' Call this function in each timers Timer-event.
'
'
Sub Dlg3DTimer (frmTarget As Form, bHasFocus As Integer)
    
    'Only redraw border if active state changes
    If bHasFocus Then
	If frmTarget.hWnd <> GetActiveWindow() Then
	    bHasFocus = False
	    DrawDlg3D frmTarget, False, 0
	End If
    Else
	If frmTarget.hWnd = GetActiveWindow() Then
	    bHasFocus = True
	    DrawDlg3D frmTarget, False, 0
	End If
    End If

End Sub

'
' DrawDlg3D paints the Form frmTarget with a 3D-border. This applies
' of course only to Forms with BorderStyle = "3 - Fixed Double".
' Set bBlaster to True if used with MsgBlaster methode. bActive is only
' required with MsgBlaster methode and should be True if Form is
' currently active.
'
' Idea and original coding was made by Dani Marthaler.
'
'
Sub DrawDlg3D (frmTarget As Form, bBlaster As Integer, bActive As Integer)
    Dim hWindowDC As Integer, hOldPen As Integer, hOldBrush As Integer, hOldFont As Integer
    Dim nOldTextAlign As Integer, nOldBkMode As Integer, nOldTextColor As Long
    Dim nLeft As Integer, nRight As Integer, nTop As Integer, nBottom As Integer
    Dim Ret As Long
    Dim hPenBlack As Integer, hPenWhite As Integer       'Pen Handles
    Dim hPenLightGray As Integer, hPenLightGray2 As Integer, hPenDarkGray As Integer
    Dim hBrushLightGray As Integer, hBrushWhite As Integer
    Dim hBrushCaptionBar As Integer, nColorCaptionText As Long
    Dim Rect As typRect
    Dim sCaption As String


    ' Make sure form is a DialogBox and has not "No 3D" in its Tag property
    If (frmTarget.BorderStyle <> 3) Or (InStr(UCase(frmTarget.Tag), "NO 3D") <> 0) Then Exit Sub

    ' Create DC and misc handles
    hWindowDC = GetWindowDC(frmTarget.hWnd)
    
    hPenBlack = CreatePen(PS_SOLID, 1, COLOR_BLACK)
    hPenWhite = CreatePen(PS_SOLID, 1, COLOR_WHITE)
    hPenLightGray = CreatePen(PS_SOLID, 1, COLOR_LIGHT_GRAY)
    hPenLightGray2 = CreatePen(PS_SOLID, 10, COLOR_LIGHT_GRAY)
    hPenDarkGray = CreatePen(PS_SOLID, 1, COLOR_DARK_GRAY)
    
    If bBlaster Then    'Following handles only needed for MsgBlaster methode
	hBrushLightGray = CreateSolidBrush(COLOR_LIGHT_GRAY)
	hBrushWhite = CreateSolidBrush(COLOR_WHITE)
	
	'Check if Form is active window
	If bActive Then
	    hBrushCaptionBar = CreateSolidBrush(GetSysColor(COLOR_ACTIVECAPTION))
	    nColorCaptionText = GetSysColor(COLOR_CAPTIONTEXT)
	Else
	    hBrushCaptionBar = CreateSolidBrush(GetSysColor(COLOR_INACTIVECAPTION))
	    nColorCaptionText = GetSysColor(COLOR_INACTIVECAPTIONTEXT)
	End If
    End If

    'Initialize misc variables
    nLeft = 0: nTop = 0
    nRight = frmTarget.Width / Screen.TwipsPerPixelX
    nBottom = frmTarget.Height / Screen.TwipsPerPixelY
    

    ' lightgray paintings
    hOldPen = SelectObject(hWindowDC, hPenLightGray2)
    Ret = LineTo(hWindowDC, nLeft, nBottom)
    Ret = LineTo(hWindowDC, nRight, nBottom)
    Ret = LineTo(hWindowDC, nRight, nTop)
    Ret = LineTo(hWindowDC, nLeft, nTop)

    ' white
    Ret = SelectObject(hWindowDC, hPenWhite)
    Ret = MoveTo(hWindowDC, nLeft + 1, nTop + 1)
    Ret = LineTo(hWindowDC, nLeft + 1, nBottom - 2) ' Left
    Ret = MoveTo(hWindowDC, nLeft + 1, nTop + 1)
    Ret = LineTo(hWindowDC, nRight - 1, nTop + 1) ' Top
    Ret = MoveTo(hWindowDC, nLeft + dlgBorderWidth - 1, nTop + dlgBorderWidth + dlgCaptionHeight - 1)
    Ret = LineTo(hWindowDC, nRight - dlgBorderWidth + 1, nTop + dlgBorderWidth + dlgCaptionHeight - 1) ' Under Captionbar
    Ret = MoveTo(hWindowDC, nRight - dlgBorderWidth, nTop + dlgBorderWidth - 1)
    Ret = LineTo(hWindowDC, nRight - dlgBorderWidth, nTop + dlgBorderWidth + dlgCaptionHeight - 1)' Right of Captionbar
    
    ' darkgray
    Ret = SelectObject(hWindowDC, hPenDarkGray)
    Ret = MoveTo(hWindowDC, nLeft, nTop)
    Ret = LineTo(hWindowDC, nLeft, nBottom) ' Left
    Ret = MoveTo(hWindowDC, nRight - 2, nTop + 2)
    Ret = LineTo(hWindowDC, nRight - 2, nBottom - 1) ' Right
    Ret = MoveTo(hWindowDC, nLeft + 1, nTop)
    Ret = LineTo(hWindowDC, nRight, nTop) ' Top
    Ret = MoveTo(hWindowDC, nLeft, nBottom - 2)
    Ret = LineTo(hWindowDC, nRight - 1, nBottom - 2) ' Bottom
    Ret = MoveTo(hWindowDC, nLeft + dlgBorderWidth - 1, nTop + dlgBorderWidth - 2)
    Ret = LineTo(hWindowDC, nRight - dlgBorderWidth + 1, nTop + dlgBorderWidth - 2)' On top of Captionbar
    Ret = MoveTo(hWindowDC, nLeft + dlgBorderWidth - 1, nTop + dlgBorderWidth - 2)
    Ret = LineTo(hWindowDC, nLeft + dlgBorderWidth - 1, nTop + dlgBorderWidth + dlgCaptionHeight - 1)' Left of Captionbar
    
    ' black
    Ret = SelectObject(hWindowDC, hPenBlack)
    Ret = MoveTo(hWindowDC, nRight - 1, nTop + 1)
    Ret = LineTo(hWindowDC, nRight - 1, nBottom) ' Right
    Ret = MoveTo(hWindowDC, nLeft, nBottom - 1)
    Ret = LineTo(hWindowDC, nRight - 1, nBottom - 1) ' Bottom
    
    
    ' Following drawing are only needed for MsgBlaster methode
    If bBlaster Then
	' SystemBox
	Rect.Left = nLeft + dlgBorderWidth
	Rect.Top = nTop + dlgBorderWidth - 1
	Rect.right = nLeft + dlgBorderWidth + dlgSysMenuWidth
	Rect.bottom = nTop + dlgBorderWidth + dlgSysMenuWidth - 1
	
	Ret = MoveTo(hWindowDC, Rect.right, Rect.Top) ' black line
	Ret = LineTo(hWindowDC, Rect.right, Rect.bottom)
	
	Ret = FillRect(hWindowDC, Rect, hBrushLightGray) ' box
	
	hOldBrush = SelectObject(hWindowDC, hBrushWhite) ' bar in box
	Ret = Rectangle(hWindowDC, Rect.Left + 2, Rect.Top + 7, Rect.right - 3, Rect.bottom - 8)
	Ret = SelectObject(hWindowDC, hPenDarkGray)
	Ret = MoveTo(hWindowDC, Rect.right - 3, Rect.Top + 8)
	Ret = LineTo(hWindowDC, Rect.right - 3, Rect.bottom - 8)
	Ret = LineTo(hWindowDC, Rect.Left + 2, Rect.bottom - 8)
	
	' Dialog caption
	Rect.Left = Rect.right + 1
	Rect.right = nRight - dlgBorderWidth
	Ret = FillRect(hWindowDC, Rect, hBrushCaptionBar)
    
	nLeft = Rect.Left + (Rect.right - Rect.Left) / 2
	sCaption = frmTarget.Caption
	nOldTextAlign = SetTextAlign(hWindowDC, TA_CENTER Or TA_TOP)
	nOldTextColor = SetTextColor(hWindowDC, nColorCaptionText)
	nOldBkMode = SetBkMode(hWindowDC, TRANSPARENT)
	hOldFont = SelectObject(hWindowDC, GetStockObject(SYSTEM_FONT))
	Ret = TextOut(hWindowDC, nLeft, Rect.Top + 1, sCaption, Len(sCaption))
	Ret = SetTextAlign(hWindowDC, nOldTextAlign)
	Ret = SetTextColor(hWindowDC, nOldTextColor)
	Ret = SetBkMode(hWindowDC, nOldBkMode)
	Ret = SelectObject(hWindowDC, hOldFont)
    
	Ret = SelectObject(hWindowDC, hOldBrush)
	Ret = DeleteObject(hBrushCaptionBar)
	Ret = DeleteObject(hBrushLightGray)
	Ret = DeleteObject(hBrushWhite)
    End If


    'clean-up
    Ret = SelectObject(hWindowDC, hOldPen)
    Ret = DeleteObject(hPenBlack)
    Ret = DeleteObject(hPenWhite)
    Ret = DeleteObject(hPenLightGray)
    Ret = DeleteObject(hPenLightGray2)
    Ret = DeleteObject(hPenDarkGray)
    Ret = ReleaseDC(frmTarget.hWnd, hWindowDC)

End Sub

'
' FormIn3D paints a 3D-border around controls on the given Form frmTarget.
' nBevel controls the the deepness of the 3D-border. bBlaster must be True if
' you use MsgBlaster instead of the Timer methode.
' FormIn3D calls also the Dlg3DFrame function which draws a 3D-border around
' the form. Also, the first time you call FormIn3D, CTL3D.DLL is initiated to
' transform MsgBoxes and Common Dialogs to 3D.
' Controls that are affected:
'       TextBox         ListBox         ComboBox
'       DriveListBox    DirListBox      FileListBox
'       Line
'       ... (list can be easly expanded)
'
' Just put "No 3D" in the Tag property of a specific control or the form
' itself and it is not painted in 3D.
'
' Call this function from your forms Paint-event.
'
' DON'T FORGET TO CALL Unregister_Ctl3D WHEN YOUR APP IS UNLOADED!
'
' Parts of this code are taken from the VB Tips & Tricks help file.
' Original code written by Matej Nastran.
'
'
Sub FormIn3D (frmTarget As Form, nBevel As Integer, bBlaster As Integer)
    Dim DrawWidthOld As Integer, ScaleModeOld As Integer
    Dim i As Integer, Ret As Integer
    Dim ctrlTarget As Control
    Static bBusy As Integer
    

    If bBusy Then Exit Sub          'Got some DoEvents. Just in case...
    bBusy = True

    DrawWidthOld = frmTarget.DrawWidth
    frmTarget.DrawWidth = 1
    ScaleModeOld = frmTarget.ScaleMode
    frmTarget.ScaleMode = 1     'Twips

    DoEvents
    
    'Draw 3D-border around a Dialog Box if the Timer methode is used
    If Not bBlaster Then
	DrawDlg3D frmTarget, False, 0
    End If

    'Loop controls
    For i = 0 To (frmTarget.Controls.Count - 1)
	Set ctrlTarget = frmTarget.Controls(i)
	If TypeOf ctrlTarget Is TextBox Then ControlIn3D ctrlTarget, nBevel, 0, True
	If TypeOf ctrlTarget Is ListBox Then ControlIn3D ctrlTarget, nBevel, 0, True
	If TypeOf ctrlTarget Is ComboBox Then   'ComboBoxes are special
	    ComboBoxIn3D ctrlTarget, nBevel
	End If
	If TypeOf ctrlTarget Is DriveListBox Then ControlIn3D ctrlTarget, nBevel, 0, True
	If TypeOf ctrlTarget Is DirListBox Then ControlIn3D ctrlTarget, nBevel, 0, True
	If TypeOf ctrlTarget Is FileListBox Then ControlIn3D ctrlTarget, nBevel, 0, True
	If TypeOf ctrlTarget Is Line Then       'Lines are also special
	    LineIn3D ctrlTarget
	End If
	'If TypeOf ctrlTarget Is Label Then ControlIn3D ctrlTarget, nBevel, 0, True
    Next i
    
    frmTarget.DrawWidth = DrawWidthOld      'Always restore what you change
    frmTarget.ScaleMode = ScaleModeOld
    

    bBusy = False

End Sub

'
' LineIn3D paints the given Line-control ctrlLine in 3D.
' frmTarget is the Form containing that Line.
'
Sub LineIn3D (ctrlLine As Control)

    If InStr(UCase(ctrlLine.Tag), "NO 3D") = 0 Then
	ctrlLine.BorderColor = COLOR_DARK_GRAY
	'Check if line is vertical or horizontal
	If Abs(ctrlLine.X2 - ctrlLine.X1) > Abs(ctrlLine.Y2 - ctrlLine.Y1) Then
	    ctrlLine.Parent.Line (ctrlLine.X1, ctrlLine.Y1 + Screen.TwipsPerPixelY)-(ctrlLine.X2, ctrlLine.Y2 + Screen.TwipsPerPixelY), COLOR_WHITE
	Else
	    ctrlLine.Parent.Line (ctrlLine.X1 + Screen.TwipsPerPixelX, ctrlLine.Y1)-(ctrlLine.X2 + Screen.TwipsPerPixelX, ctrlLine.Y2), COLOR_WHITE
	End If
    End If

End Sub

